/****************************************************************************\
*   FILEIO - this source file is part of Binary vIEW project (BIEW)          *
*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*
*   Copyrights:           1995, 2000 Nick Kurshev                            *
*   License:              See below                                          *
*   Author and developer: Nick Kurshev                                       *
*   Requirement:          ANSI C compatible development system               *
*   Original file name:   lib/sysdep/i86/os2/fileio.c                        *
*---------------------- [ D e s c r i p t i o n ] ---------------------------*
*  This file contain implementation of file i/o routines for OS/2.           *
*-------------------------- [ L i c e n c e ] -------------------------------*
* The Binary vIEW (BIEW) is copyright (C) 1995 Nick Kurshev.                 *
* All rights reserved. This software is redistributable under the licence    *
* given in the file "Licence.en" ("Licence.ru" in russian translation)       *
* distributed in the BIEW archive.                                           *
\****************************************************************************/
#include <os2.h>
#include <string.h>
#include <stdio.h>
#include <errno.h>
#include "_sys_dep.h"

static char f_buff[FILENAME_MAX+1];

static char * __NEAR__ fnUnix2Dos(const char *fn)
{
  char *ptr;
  strncpy(f_buff,fn,sizeof(f_buff)-1);
  ptr = f_buff;
  while((ptr = strchr(ptr,'/')) != NULL) *ptr = '\\';
  return f_buff;
}

void __OsClose(int handle)
{
  errno = DosClose(handle);
}

int __OsDelete(const char *name)
{
  char *dosname;
  dosname = fnUnix2Dos(name);
  return errno = DosDelete((PSZ)dosname,0);
}

int __OsRename(const char *oldname,const char *newname)
{
  char olddosname[FILENAME_MAX+1],*newdosname;
  newdosname = fnUnix2Dos(oldname);
  strncpy(olddosname,newdosname,sizeof(olddosname)-1);
  newdosname = fnUnix2Dos(newname);
  return errno = DosMove((PSZ)olddosname,(PSZ)newdosname,0);
}

int  __OsDupHandle(int handle)
{
  USHORT ret;
  ret = -1; /* need by OS/2 SDK */
  errno = DosDupHandle(handle,&ret);
  if(errno) ret = -1;
  return ret;
}

int __OsChSize(int handle, long size)
{
  errno = DosNewSize(handle,size);
  return errno ? -1 : 0;
}

int  __OsCreate(char *fname)
{
   USHORT fhandle,action,rc;
   char *dosname;
   dosname = fnUnix2Dos(fname);
   rc = DosOpen((PSZ)dosname,&fhandle,&action,0L,0,
                OPEN_ACTION_CREATE_IF_NEW | OPEN_ACTION_FAIL_IF_EXISTS,
                OPEN_ACCESS_READWRITE | OPEN_SHARE_DENYREADWRITE,0L);
   if(rc) { fhandle = -1; errno = rc; }
   return fhandle;
}

int  __OsOpen(char *fname,int mode)
{
   USHORT fhandle,action,rc;
   char *dosname;
   dosname = fnUnix2Dos(fname);
   mode |= OPEN_FLAGS_FAIL_ON_ERROR;
           /* report media i/o error trough retcode but not
              through critical error */
   rc = DosOpen((PSZ)dosname,&fhandle,&action,0L,0,
                OPEN_ACTION_OPEN_IF_EXISTS,mode,0L);
   if(rc) { fhandle = -1; errno = rc; }
   return fhandle;
}

long __OsSeek( int handle, long offset, int origin)
{
  unsigned long ret;
  errno = DosChgFilePtr(handle,offset,origin,&ret);
  if(errno) ret = -1;
  return ret;
}

int __OsTruncFile( int handle, unsigned long size)
{
  return DosNewSize(handle,size);
}

int __OsRead(int handle, void *buff,unsigned size)
{
  USHORT ret;
  errno = DosRead(handle,buff,size,&ret);
  return ret;
}

int __OsWrite(int handle, void *buff,unsigned size)
{
  USHORT ret;
  errno = DosWrite(handle,buff,size,&ret);
  return ret;
}
